#!/usr/local/bin/perl

# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
#
# SPPDレンタルサーバー フォームメールCGI Ver1.15 ログ表示/DL
# (C) 2004-2017 Playing Network ,INC. All Rights Reserved.
# Programmed By Yasushi Toguchi
# 
# 著作権は株式会社プレイングネットワークに帰属します。
#
# -=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=

sub maillog_action
{
	use Encode;
	use POSIX;
	my $layout = $url{'m_action'} == 2 ? $template_files."maillog_del.html" : $template_files."maillog_action.html";
	my $page = $url{'page'} && $url{'page'} =~ /^\d+$/ ? $url{'page'} : 1;

	#----------------#
	# ■引数チェック #
	#----------------#
	&ErrorMessage('開始"年"が不正です。') if($url{s_year} !~ /^\d{4}$/);
	&ErrorMessage('開始"月"が不正です。') if($url{s_mon} !~ /^\d{2}$/);
	&ErrorMessage('開始"日"が不正です。') if($url{s_day} !~ /^\d{2}$/);
	&ErrorMessage('終了"年"が不正です。') if($url{e_year} !~ /^\d{4}$/);
	&ErrorMessage('終了"月"が不正です。') if($url{e_mon} !~ /^\d{2}$/);
	&ErrorMessage('終了"日"が不正です。') if($url{e_day} !~ /^\d{2}$/);
	&ErrorMessage('表示件数を10～99の範囲で指定してください。') if($url{maxrows} !~ /^\d{2}$/);

	#------------#
	# ■検索範囲 #
	#------------#
	$S_YM  = $url{s_year}.$url{s_mon}.$url{s_day};
	$E_YM  = $url{e_year}.$url{e_mon}.$url{e_day};

	#------------#
	# ■検索実行 #
	#------------#
	opendir (DIR,"$LogDir");
	if($url{type} eq "1")
	{
		#ダウンロードしてないファイル
		@LogList = sort grep (/^\d+\.log$/ && (-f "$LogDir"."$_") && ($S_YM <= substr($_,0,8)) && ($E_YM >= substr($_,0,8)),readdir (DIR));
	}
	elsif($url{type} eq "2")
	{
		#全て
		@LogList = sort grep (/^\d+\.(log|txt)$/ && (-f "$LogDir"."$_") && ($S_YM <= substr($_,0,8)) && ($E_YM >= substr($_,0,8)),readdir (DIR));
	}
	else
	{
		#ダウンロードしたファイル
		@LogList = sort grep (/^\d+\.txt$/ && (-f "$LogDir"."$_") && ($S_YM <= substr($_,0,8)) && ($E_YM >= substr($_,0,8)),readdir (DIR));
	}
	closedir (DIR);

	#--------------------------#
	# ■該当ファイルが無い場合 #
	#--------------------------#
	&ErrorMessage('条件に該当するファイルがありません。') if(scalar(@LogList) <= 0);

	#----------------------#
	# ■フィールド名リスト #
	#----------------------#
	@fieldname=();
	foreach $filename (@LogList)
	{
		open(LOG,$LogDir.$filename);
		while(<LOG>)
		{
			$. <= 2 ? next : chop;
			my $name = (split('='))[0];
#			push(@fieldname,$name) if(!grep(/^$name$/,@fieldname));
			push(@fieldname,$name) if(!&in_array($name, @fieldname));
		}
		close(LOG);
	}

	#--------------------------------#
	# ■ダウンロード or 表示 or 削除 #
	#--------------------------------#
	if($url{'m_action'} == 1)
	{
		#----------------------------------#
		# ■検索に該当したファイルを       #
		#   1つのCSVにまとめてダウンロード #
		#----------------------------------#
		print "Content-type: application/octet-stream; name=\"formmail.csv\"\n";
		print "Content-Disposition: attachment;filename=\"formmail.csv\"\n\n";

		# フィールド名表示 #
		my $header = $Quoto."日付".$Quoto.$Delimiter.$Quoto."時間".$Quoto;
		foreach $field (@fieldname)
		{
			$header .= $Delimiter.$Quoto.$field.$Quoto;
		}
		Encode::from_to($header, "utf-8", "cp932");
		print "$header\r\n";

		foreach $filename (@LogList)
		{
			my $Fullfilename = my $FullfilenameMv = "$LogDir"."$filename";
			$FullfilenameMv =~ s/log$/txt/i;

			open(MAILLOG,"$Fullfilename");
			my(@Maildata) = <MAILLOG>;
			close(MAILLOG);
			chop(@Maildata);

			my($Maildata_Tmp) = $Quoto.$Maildata[0].$Quoto.$Delimiter.$Quoto.$Maildata[1].$Quoto;
			foreach $field (@fieldname)
			{
				# 該当フィールドのデータ取り出し
				my $field_data = "";
				foreach my $m_line (@Maildata) {
					if(index($m_line, $field) eq 0) {
						$field_data .= (split('=', $m_line, 2))[1]."<br>";
					}
				}
				$field_data =~ s/<br>/$BRto/ig;
				$Maildata_Tmp .= ($field_data ? $Delimiter.$Quoto.$field_data.$Quoto : $Delimiter.$Quoto.$Quoto);
			}
			Encode::from_to($Maildata_Tmp, "utf-8", "cp932");
			print "$Maildata_Tmp\r\n";

			rename($Fullfilename, $FullfilenameMv);
		}
	}
	elsif($url{'m_action'} == 2)
	{
		# ■削除画面 #
		print "Content-type: text/html\n\n";
		my $maxpage = ceil(scalar(@LogList)/$url{'maxrows'});
		@LogList = splice(@LogList,($page-1)*$url{'maxrows'},$url{'maxrows'});

		open(HTML,"$layout");
		while(<HTML>)
		{
			if(/(.*)(\%PAGELIST\%)(.*)/)
			{
				print $1;

				# ページリスト表示 #
				for(my($i)=1;$i<=$maxpage;$i++)
				{
					print $i == $page ? " $i\n" : " <a href=\"$ENV{'SCRIPT_NAME'}?action=$url{'action'}&s_year=$url{'s_year'}&s_mon=$url{'s_mon'}&s_day=$url{'s_day'}&e_year=$url{'e_year'}&e_mon=$url{'e_mon'}&e_day=$url{'e_day'}&maxrows=$url{'maxrows'}&type=$url{'type'}&m_action=$url{'m_action'}&page=$i\">$i</a>\n";
				}

				print $3;
			}
			elsif(/(.*)(\%MAILLIST\%)(.*)/)
			{
				print $1;
				# フィールド名表示 #
				print "<tr bgcolor=\"#e2e2e2\">\n\t<th nowrap>&nbsp;</th>\n\t<th nowrap>日付</th>\n\t<th nowrap>時間</th>\n";
				foreach $field (@fieldname)
				{
					print "\t<th nowrap>$field</th>\n";
				}
				print "</tr>\n";

				# データ表示 #
				foreach $filename (@LogList)
				{
					my $Fullfilename = "$LogDir"."$filename";
					open(MAILLOG,"$Fullfilename");
					my(@Maildata) = <MAILLOG>;
					close(MAILLOG);
					chop(@Maildata);

					print qq(<tr>\n\t<td><input type="checkbox" name="files" value="$filename"></td>\n\t<td>$Maildata[0]</td>\n\t<td>$Maildata[1]</td>\n);
					foreach $field (@fieldname)
					{
						# 該当フィールドのデータ取り出し
						my $field_data = "";
						foreach my $m_line (@Maildata) {
							if(index($m_line, $field) eq 0) {
								$field_data .= (split('=', $m_line, 2))[1]."\n";
							}
						}

						$field_data =~ s/</&lt;/g;
						$field_data =~ s/>/&gt;/g;
						$field_data =~ s/&lt;br&gt;/<br>/ig;
						$field_data =~ s/\n/<br>/ig;

						print "\t<td>";
						print $field_data ?  $field_data : "&nbsp;";
						print "</td>\n";
					}
					print "</tr>\n";
				}
				print $3;
			}
			else
			{
				print;
			}
		}
	}
	else
	{
		# ■表示画面 #
		print "Content-type: text/html\n\n";
		my $maxpage = ceil(scalar(@LogList)/$url{'maxrows'});
		@LogList = splice(@LogList,($page-1)*$url{'maxrows'},$url{'maxrows'});

		open(HTML,"$layout");
		while(<HTML>)
		{
			if(/(.*)(\%PAGELIST\%)(.*)/)
			{
				print $1;

				# ページリスト表示 #
				for(my($i)=1;$i<=$maxpage;$i++)
				{
					print $i == $page ? " $i\n" : " <a href=\"$ENV{'SCRIPT_NAME'}?action=$url{'action'}&s_year=$url{'s_year'}&s_mon=$url{'s_mon'}&s_day=$url{'s_day'}&e_year=$url{'e_year'}&e_mon=$url{'e_mon'}&e_day=$url{'e_day'}&maxrows=$url{'maxrows'}&type=$url{'type'}&m_action=$url{'m_action'}&page=$i\">$i</a>\n";
				}

				print $3;
			}
			elsif(/(.*)(\%MAILLIST\%)(.*)/)
			{
				print $1;

				# フィールド名表示 #
				print "<tr bgcolor=\"#e2e2e2\">\n\t<th nowrap>日付</th>\n\t<th nowrap>時間</th>\n";
				foreach $field (@fieldname)
				{
					print "\t<th nowrap>$field</th>\n";
				}
				print "</tr>\n";

				# データ表示 #
				foreach $filename (@LogList)
				{
					my $Fullfilename = "$LogDir"."$filename";
					open(MAILLOG,"$Fullfilename");
					my(@Maildata) = <MAILLOG>;
					close(MAILLOG);
					chop(@Maildata);

					print "<tr>\n\t<td>$Maildata[0]</td>\n\t<td>$Maildata[1]</td>\n";
					foreach $field (@fieldname)
					{
						# 該当フィールドのデータ取り出し
						my $field_data = "";
						foreach my $m_line (@Maildata) {
							if(index($m_line, $field) eq 0) {
								$field_data .= (split('=', $m_line, 2))[1]."\n";
							}
						}

						$field_data =~ s/</&lt;/g;
						$field_data =~ s/>/&gt;/g;
						$field_data =~ s/&lt;br&gt;/<br>/ig;
						$field_data =~ s/\n/<br>/ig;

						print "\t<td>";
						print $field_data ?  $field_data : "&nbsp;";
						print "</td>\n";
					}
					print "</tr>\n";
				}

				print $3;
			}
			else
			{
				print;
			}
		}
	}
}

#--------------#
# ●メール削除 #
#--------------#
sub maildel
{
	undef(@files);
	local($i)="";

	if($ENV{'REQUEST_METHOD'} eq "POST")
	{
		foreach(split('&',$str))
		{
			my($variable,$value)=split('=',$_);
			$value=~ tr/+/ /;
			$value=~ s/%([0-9a-fA-F][0-9a-fA-F])/pack("C",hex($1))/eg;
			push(@files,$value) if($variable == "files");
		}

		foreach $filename (@files)
		{
			next if($filename !~ /^\d+\.(log|txt)$/);
			$Fullfilename = "$LogDir"."$filename";
			unlink($Fullfilename);
			$i="yes";
		}
	}
	print "Location: ./index.cgi?deleted=$i\n\n";
	exit;
}

sub in_array {
	my $value = shift;
	my @array = @_;

	foreach(@array) {
		return 1 if($_ eq $value);
	}

	return 0;
}

1
